% this program produces Appendix figure 1
% Note this program can also be used for subsidy counterfactuals, which is dropped in revision process.
clear all;diary appendix_figure1;

display('******************************************************')
display('* Counterfactual Results for ed10: appendix figure 1 *')
display('******************************************************')

% step 1: load data and estimated parameters

global M J N N1 K R KX KZ typeK n_theta X Z entry citymatch match eita Nr;

load randomeffects2/voice2_a_ed10_baseline;
load randomeffects2/data/draws_symmetric.mat; 
data=voice2_a_ed10_baseline;

global M J N N1 K R KX KZ typeK n_theta X Z entry citymatch match eita Nr;
K = 8 % max type = 8. There are 9 types of players from type 0 to 8.
typeK = linspace(0,K,K+1);
N = size(data,1); 

% use full sample. Can use a subsample to debug program.
N1 = N

disp('X=[1 pop/1000/1000  pct_black median_age hhsize pct_frgn_born inc/1000 pct_poverty gte rbc log(estab/1000) size_estab pct_manuf]')
X = [ones(N1,1)/10 data(1:N1,6)/1000 data(1:N1,7) data(1:N1,8)/100 data(1:N1,9)/10 data(1:N1,10) data(1:N1,11)/100 data(1:N1,14) data(1:N1,16:17)/10 log(data(1:N1,18)) data(1:N1,19)/100 data(1:N1,20)];
KX = size(X,2);

age_firm = data(1:N1,23); 
subsidiary = data(1:N1,24); 
private = data(1:N1,21); 
venture_capital= data(1:N1,22);
exp_m = data(1:N1,42); 
age_manager = data(1:N1,43); 
has_graddegree = data(1:N1,44); 
ug_usnew25 = data(1:N1,46); 
sat1400 = (data(1:N1,50) >= 1400); 
has_econbus = data(1:N1,51);
has_econbus5 = data(1:N1,52);
has_engineer = data(1:N1,53);

ln_exp_m = log(exp_m+1); ln_age_manager = log(age_manager+1);
ln_age_firm = log(age_firm + 1); age_firm_3_10 = (age_firm >= 3) .* (age_firm < 10); age_firm_above10 = (age_firm >=10);
i_exp_econ = ln_exp_m .* has_econbus5;

disp('Z = [1 ln_exp_m sat1400 has_econbus5 i_exp_econ has_engineer has_graddegree ln_age_firm subsidiary private venture_capital]')
Z =[ ones(N1,1) ln_exp_m sat1400 has_econbus5 i_exp_econ has_engineer has_graddegree ln_age_firm subsidiary private venture_capital]/10;
KZ = size(Z,2);

scale = 1./[10 1 1 100 10 1 100 1 10 10 1 100 1 1 1 10*ones(1, size(Z,2))]';

entry = data(1:N1,5);
city_ID = data(1:N1,1);

M = max(city_ID);	%  # of cities where CLECs can potentially enter
citymatch = sparse(zeros(N1,M));
city_temp = sparse(ones(N1,1) * linspace(1, M, M));
citymatch = logical(city_temp == city_ID*ones(1,M));
match = citymatch * citymatch' - eye(N1);
clear city_temp;

% draws_symmetric.mat: a data set which is a random drawn matrix (M*Nr) from normal(0,1) and transformed it to match firm-market-year observations. 
Nr = 20 
eita_all = citymatch * eita_m;
eita = eita_all(1:N1,1:Nr);

% parameters to estimate: beta, sigma, phi and gama.
n_theta = KX + 2 + KZ;

theta_hat=[3.3298;2.0587;2.5770;-0.1087;-2.3464;4.1145;-0.0045;7.2346;1.9644;1.1960;1.9818;0.0470;-3.4778;0.7956;-0.6549;0.6005;0.1609;0.0685;0.3960;-0.1649;-0.0780;0.0294;0.0452;-0.1383;-0.1285;-0.0052];

theta_hat=theta_hat./scale;

tau_estimated=data(1:N1,62);


% step 2: simulate entry and entry regret under estimated parameters to corroborate later simulation.

NS=50
espilon=zeros(N1,NS);

for ns=1:NS,
    
    epsilon(:,ns)=normrnd(0,1,N1,1);

    for h=1:Nr;
    X1=[X eita(:,h)];
 
    expost_profit_h(:,h,ns)=X1*theta_hat(1:(KX+1))+match*entry*theta_hat(KX+2)+epsilon(:,ns);
    expost_regret1_h(:,h,ns)=((entry==1).*(expost_profit_h(:,h,ns)<0)==1);
    expost_regret2_h(:,h,ns)=((entry==1)+(expost_profit_h(:,h,ns)<0)==0);
    expost_regret_h(:,h,ns)=expost_regret1_h(:,h,ns)+expost_regret2_h(:,h,ns);
    
    end;

     expost_profit(:,ns)=mean(expost_profit_h(:,:,ns),2);
     expost_regret1(:,ns)=mean(expost_regret1_h(:,:,ns),2);
     expost_regret2(:,ns)=mean(expost_regret2_h(:,:,ns),2);
     expost_regret(:,ns)=mean(expost_regret_h(:,:,ns),2);
     
end;

display('*entry and entry regret under estimated parameters*')
display('*mean_tau_estimated entry_rate mean(expost_regret1) mean(expose_regret2) mean(expost_regret) mean(expost_profit)*')
mean_regret=[mean(expost_regret1,2) mean(expost_regret2,2) mean(expost_regret,2) mean(expost_profit,2)];
[mean(tau_estimated) mean(entry) mean(mean_regret)]

tau_addon=[-1000 -1.757 -1.257 -0.757 -0.257 0.243 0.743 1.243 1.743 2.243];

S=size(tau_addon,2);

tic
display('*add 0 thousand business establishments to every market*')
exp_tauchange3; regret0=regret; entry_tau0=entry_tau_overns;
display('mean_tau entry_rate mean(expost_regret1) mean(expose_regret2) mean(expost_regret)*')
[mean_tau regret0(:,1:4)]
toc

%data=[data entry_tau0 zero_entrants_overns one_entrants_overns five_entrants_overns];
%save counterfactual_subsidy3 data -ASCII -TABS

diary off
