% Basically, this function is a replication of logL_g_r1.m. The only
% difference is that we get individual gradients in order to calculate 
% variance covariance matrix using BHHH estimator (Greene P481).

function [se_theta,zmle,Pmle]=func_var(theta0)

% eita is a N1 by Nr matrix simulated for unobserved market level heterogeneity. Nr is the number of draws. 
global M J N N1 K KX KZ R typeK n_theta X Z entry citymatch match Nr eita;

% Martix to store market likelihoods (1:M) for each simulation (1:Nr). Need
% to store the likelihoods for the nodes other than the current one, since
% they are used in the computation of the gradients
Lall = zeros(1,M); Lalp = zeros(Nr,M); La = zeros(N1,Nr);
 
%initializes gradient for each coefficent       
gbeta=zeros(KX+1,M); ggama=zeros(1,M); gra1=zeros(KZ,M);gra2=zeros(KZ,M);

% create tau parameter (only needed for type 2 and above) 
% exp() to ensure that tau is non-negative
% note exp() increases really fast, and poisspdf(0,a very large number)=0 (rounding error in matlab)
tau=exp(Z*theta0((KX+3):n_theta));
fi(:,1)=tau.^(0)/factorial(0);
fi_firm(:,K+1)=tau.^(K)/factorial(K);

for h=1:Nr;
    
    X1=[X eita(:,h)];
    %type 0's latent profit, without considering opponents' entry decisions
    profit_k0=X1*theta0(1:KX+1); 
    normcdf_profit_k0=normcdf(profit_k0);
    normpdf_profit_k0=normpdf(profit_k0);

    %type 1's profit function
    % citycitymatch'*normcdf(profit_k0): a M*1 vector, expected type-0 entrants for all M cities
    % citymatch*citymatch'*normcdf(profit_k0); N1*1, expected type-0 entrants into the city
    % need to substract entering probablity of itself
    A=match*normcdf_profit_k0;
    profit_k1=profit_k0+A*theta0(KX+2);

    profit=[profit_k0 profit_k1 zeros(N1,K-1)];

    % for calculating gradients
    gb1=ones(N1,1)+(match*normpdf_profit_k0)*theta0(KX+2);
    gg1=A;

    gb=[ones(N1,1) gb1 zeros(N1,K-1)];
    gg=[zeros(N1,1) gg1 zeros(N1,K-1)];
    gr=zeros(N1,K+1,KZ);

 for k=2:K,
    
    % a type k player thinks that there are k types of rivals (0, 1, to k-1) in the market 
    type=linspace(0,k-1,k);
    % f: type k's perceived distribution about rivals, from truncated poisson, a k*N1 matrix
    fi(:,k)=tau.^(k-1)/factorial(k-1);
     
    sum_fi=fi*ones(k,k);
    f=fi./sum_fi;
    
    %type k's profit function
    exptd_pr_rvls_k=normcdf(sum(profit(:,1:k).*f,2)); 
    A_k=match*exptd_pr_rvls_k;
    profit(:,k+1)=profit_k0+A_k*theta0(KX+2);    

    B=normpdf(sum(profit(:,1:k).*f,2));
    exptd_gb_k=sum(gb(:,1:k).*f,2).*B;
    gb(:,k+1)=ones(N1,1)+match*exptd_gb_k*theta0(KX+2);    
    
    exptd_gg_k=sum(gg(:,1:k).*f,2).*B;
    gg(:,k+1)=A_k+match*exptd_gg_k*theta0(KX+2);
    
    % for df_dr_k the derivative of exp() and Z is incorporated by chain rule
    df_dr_k=fi.*(ones(N1,1)*type);
    df_dr_k=df_dr_k./sum_fi-fi.*(df_dr_k*ones(k,k))./(sum_fi.^2);

    for zk=1:KZ,           
        exptd_gr_k=sum(gr(:,1:k,zk).*f+profit(:,1:k).*(df_dr_k.*(Z(:,zk)*ones(1,k))),2).*B;
        gr(:,k+1,zk)=match*exptd_gr_k*theta0(KX+2);    
    end;
    
    clear type f exptd_pr_rvls_k A_k B exptd_gb_k  exptd_gg_k exptd_gr_k df_dr_k;

end;
    
   fi_firm(:,1:K)=fi;
   clear fi;
   sum_fi_firm=fi_firm*ones(K+1,K+1);
   f_firm=fi_firm./sum_fi_firm;

pr_in=normcdf(sum(profit.*f_firm,2));

La(:,h)=pr_in.*entry+(1-pr_in).*(1-entry);
La(:,h)=La(:,h)+0.00000001*(La(:,h)==0);

Lac=La(:,h)*ones(1,M).*citymatch; %creates (N1,M) matrix. For each market M, assigns corresponding likilihood and assigns zero to other entries
Lal=Lac+ones(N1,M)-citymatch; %For each market M, replaces all zero entries (other than cdfs) with 1 so that in the next step 
                          %can calculate product of observations for each county
Lalp(h,:)=prod(Lal);   %product of cdfs of each observation in a county
Lall=Lall+Lalp(h,:)/Nr; %sums likelihood of each county over the simulations weighted by 1/Nr
Lall=Lall+0.00000001*(Lall==0);

d_common=2*entry-1;
B_K=normpdf(sum(profit.*f_firm,2));

gbb=sparse((citymatch*Lalp(h,:)')./La(:,h)); %Inside the gradient formula, in each market, pdf for observation j is multiplied with the cdfs of all other 
                              %observation other than j in that market.And gb(j) is the multiplication of all cdfs other than j. Note that
                              %Lalp is a (1,M) vector containing all products of cdfs in each market and La is a (N1,1) vector of cdfs for each
                              %observation, and citymatch is a (N1,M) matrix of indicators for markets                             
                            
gbc=(sum(gb.*f_firm,2).*B_K.*d_common).*gbb; %multiplies pdf of j with corresponding cdf product other than cdf of j 
gbd=gbc*ones(1,M).*citymatch; %divides value of gbc into categories of counties, note that gbd is (N1,M) matrix
gbe=X1'*gbd; %gradient for beta includes this X matrix, here gbe is (KX,M) matrix
gbeta=gbeta+gbe/Nr; %gradients multiplied by weights and summed over nodes

% repeat the above procedure to derive gradients for gama and r
ggc=(sum(gg.*f_firm,2).*B_K.*d_common).*gbb; 
ggama=ggama+ones(N1,1)'*(ggc*ones(1,M).*citymatch)/Nr; 

df_dr=fi_firm.*(ones(N1,1)*typeK);
df_dr=df_dr./sum_fi_firm-fi_firm.*(df_dr*ones(K+1,K+1))./(sum_fi_firm.^2);

for zk=1:KZ,
    grc1=(sum(gr(:,:,zk).*f_firm,2).*B_K.*d_common).*gbb;
    gra1(zk,:)=gra1(zk,:)+ones(N1,1)'*(grc1*ones(1,M).*citymatch)/Nr; 
end;
grc2=(sum(profit.*df_dr,2).*B_K.*d_common).*gbb;
gra2=gra2+Z'*(grc2*ones(1,M).*citymatch)/Nr; 
gra=gra1+gra2;

end; 

L=-sum(log(Lall)); 

gbeta=gbeta./(ones(KX+1,1)*Lall);
ggama=ggama./Lall;
gra=gra./(ones(KZ,1)*Lall);

g=[-gbeta;-ggama;-gra];

%inform = 1/N*(g*g'); this is the information matrix
inform = g*g';  % for the asymptotic variance of theta, the 1/N cancelled out, var(theta)=inv(inform)/N
se_theta=sqrt(diag(inv(inform)));
zmle=theta0./se_theta;
Pmle=2*(1-normcdf(abs(zmle)));




    
