 %LIKELIHOOD FUNCTION

function L=logL_r1(theta0)

% eita is a N1 by Nr matrix simulated for unobserved market level heterogeneity. Nr is the number of draws. 
global M J N N1 K KX KZ R typeK n_theta X Z entry citymatch match Nr eita;

% Martix to store market likelihoods (1:M) for each simulation (1:Nr). Need
% to store the likelihoods for the nodes other than the current one, since
% they are used in the computation of the gradients
Lall = zeros(1,M); Lalp = zeros(Nr,M); La = zeros(N1,Nr);
 
% create tau parameter (only needed for type 2 and above) 
% exp() to ensure that tau is non-negative
% note exp() increases really fast, and poisspdf(0,a very large number)=0 (rounding error in matlab)
tau=exp(Z*theta0((KX+3):n_theta));
fi(:,1)=tau.^(0)/factorial(0);
fi_firm(:,K+1)=tau.^(K)/factorial(K);

for h=1:Nr;
    
    X1=[X eita(:,h)];
    %type 0's latent profit, without considering opponents' entry decisions
    profit_k0=X1*theta0(1:KX+1); 
    normcdf_profit_k0=normcdf(profit_k0);
    normpdf_profit_k0=normpdf(profit_k0);

    %type 1's profit function
    % citycitymatch'*normcdf(profit_k0): a M*1 vector, expected type-0 entrants for all M cities
    % citymatch*citymatch'*normcdf(profit_k0); N1*1, expected type-0 entrants into the city
    % need to substract entering probablity of itself
    A=match*normcdf_profit_k0;
    profit_k1=profit_k0+A*theta0(KX+2);

    profit=[profit_k0 profit_k1 zeros(N1,K-1)];

 for k=2:K,
    
    % a type k player thinks that there are k types of rivals (0, 1, to k-1) in the market 
    type=linspace(0,k-1,k);
    % f: type k's perceived distribution about rivals, from truncated poisson, a k*N1 matrix
    fi(:,k)=tau.^(k-1)/factorial(k-1);
     
    sum_fi=fi*ones(k,k);
    f=fi./sum_fi;
    
    %type k's profit function
    exptd_pr_rvls_k=normcdf(sum(profit(:,1:k).*f,2)); 
    A_k=match*exptd_pr_rvls_k;
    profit(:,k+1)=profit_k0+A_k*theta0(KX+2);    
end;
    
   fi_firm(:,1:K)=fi;
   clear fi;
   sum_fi_firm=fi_firm*ones(K+1,K+1);
   f_firm=fi_firm./sum_fi_firm;

pr_in=normcdf(sum(profit.*f_firm,2));

La(:,h)=pr_in.*entry+(1-pr_in).*(1-entry);
La(:,h)=La(:,h)+0.00000001*(La(:,h)==0);

Lac=La(:,h)*ones(1,M).*citymatch; %creates (N1,M) matrix. For each market M, assigns corresponding likilihood and assigns zero to other entries
Lal=Lac+ones(N1,M)-citymatch; %For each market M, replaces all zero entries (other than cdfs) with 1 so that in the next step 
                          %can calculate product of observations for each county
Lalp(h,:)=prod(Lal);   %product of cdfs of each observation in a county
Lall=Lall+Lalp(h,:)/Nr; %sums likelihood of each county over the simulations weighted by 1/Nr

end; 

L=-sum(log(Lall+0.00000001*(Lall==0))); 

