% MAIN PROGRAM. Last updated: June 2010 for AER revision 2

clear all; diary batch_spec00a_B_09012010;

display('**********************************************')
display('*Results: Year 1998, random effects, baseline*')
display('**********************************************')

load ../data/voice2_a_imputed_ed10.mat;
load ../data/draws_symmetric.mat; 

display('* *')
display('*baseline, different functiontional form for tau *')


global M J N N1 K R KX KZ typeK n_theta X Z entry citymatch match eita Nr;
R=8;
K = 8 % max type = 8. There are 9 types of players from type 0 to 8.
typeK = linspace(0,K,K+1);
N = size(data,1); 

% use full sample. Can use a subsample to debug program.
N1 = N

disp('X=[1 pop/1000/1000  pct_black median_age hhsize pct_frgn_born inc/1000 pct_poverty gte rbc log(estab/1000) size_estab pct_manuf]')
X = [ones(N1,1)/10 data(1:N1,6)/1000 data(1:N1,7) data(1:N1,8)/100 data(1:N1,9)/10 data(1:N1,10) data(1:N1,11)/100 data(1:N1,14) data(1:N1,16:17)/10 log(data(1:N1,18)) data(1:N1,19)/100 data(1:N1,20)];
KX = size(X,2);

age_firm = data(1:N1,23); 
subsidiary = data(1:N1,24); 
private = data(1:N1,21); 
venture_capital= data(1:N1,22);
exp_m = data(1:N1,42); 
age_manager = data(1:N1,43); 
has_graddegree = data(1:N1,44); 
ug_usnew25 = data(1:N1,46); 
sat1400 = (data(1:N1,50) >= 1400); 
has_econbus = data(1:N1,51);
has_econbus5 = data(1:N1,52);
has_engineer = data(1:N1,53);

ln_exp_m = log(exp_m+1); ln_age_manager = log(age_manager+1);
ln_age_firm = log(age_firm + 1); age_firm_3_10 = (age_firm >= 3) .* (age_firm < 10); age_firm_above10 = (age_firm >=10);
i_exp_econ = ln_exp_m .* has_econbus5;

disp('Z = [1 ln_exp_m sat1400 has_econbus5 i_exp_econ has_engineer has_graddegree ln_age_firm subsidiary private venture_capital]')
Z =[ ones(N1,1) ln_exp_m sat1400 has_econbus5 i_exp_econ has_engineer has_graddegree ln_age_firm subsidiary private venture_capital]/10;
KZ = size(Z,2);

scale = 1./[10 1 1 100 10 1 100 1 10 10 1 100 1 1 1 10*ones(1, size(Z,2))]';

entry = data(1:N1,5);
city_ID = data(1:N1,1);

M = max(city_ID);	%  # of cities where CLECs can potentially enter
citymatch = sparse(zeros(N1,M));
city_temp = sparse(ones(N1,1) * linspace(1, M, M));
citymatch = logical(city_temp == city_ID*ones(1,M));
match = citymatch * citymatch' - eye(N1);
clear city_temp;

Nr = 20 
eita_all = citymatch * eita_m;
eita = eita_all(1:N1,1:Nr);

% parameters to estimate: beta, sigma, phi and gama.
n_theta = KX + 2 + KZ;

theta0 = [31.807;-2.465;3.440;-11.870;-22.835;2.844;0.302;10.364;22.026;13.363;2;5.657;-3.088;0.857;-1.012;-5.516;1.371;0.620;3.254;-1.385;-0.677;0.332;0.427;-1.116;-1.069;-0.012];

tau=R*normcdf(Z*theta0((KX+3):n_theta));
tau_initial=[min(tau) mean(tau) median(tau) max(tau) std(tau)]

tic
logL_r1(theta0)
toc
tic
logL_g_r1(theta0)
toc

%theta1=theta0+0.01*randn(size(theta0)); 
% Tried different starting values for ensure global min
theta1 = theta0;

options = optimset('Display', 'iter','TolX',0.001,'TolFun',1e-6,'MaxFunEvals',50000,'MaxIter',50000,'GradObj','on','LargeScale','on','FunValCheck','on','DerivativeCheck','off'); 
tic
[theta,fval,exitflag,output,grad] = fminunc(@logL_g_r1,theta1,options) 
% tried different search algorithms to ensure global min
%[theta,fval,exitflag,output]=fminsearch(@logL_r1,theta1,options) 
toc

tau=R*normcdf(Z*theta((KX+3):n_theta));
tau_show=[min(tau) mean(tau) median(tau) max(tau) std(tau)]

% calculate the standard errors of theta
[se_theta,tmle,Pmle] = func_var(theta);
theta2 = theta .* scale;
se_theta = se_theta .* scale;

% report results
results_ed10 = [theta2 se_theta tmle Pmle]
theta_ed10 = theta2;tau_ed10 = tau;pr_in_ed10 = indv_pr_in(theta); 
likelihd_ed10 =-logL_r1(theta);  
data_tau =[data tau_ed10 pr_in_ed10];

save voice2_a_ed10_spec00a_B data_tau -ASCII -TABS;

clear data_tau M J N N1 K typeK options n_theta X entry city_ID citymatch exitflag fval output theta_g se_theta tmle Pmle;
save batch_results_spec00a_B

diary off
